% Calculate lookup table: MD vs cylinder R vs waveform
% The table is used for fitting and is saved in the *data* folder.

% powder averaging is done both theoretically for axisymmetric D and
% numerically by using averaging rotations given in spherical coordinates

clear all
close all

save_wfm_common = 0; % only need this ones for a new set of waveforms - collect spectral trace and thresholded spectra
trPS_thresh = 0.999;

restoredefaultpath
addpath(genpath(fullfile(pwd,'add')));

data_dir = fullfile(pwd,'data');

waveform_dir = 'wfm_preclinical';
waveform_names = {'PGSE_LTE', ...
    'M0_LTE_1', 'M0_LTE_2', 'M0_LTE_3', 'M0_STE', ...
    'M1_LTE_1', 'M1_LTE_2', 'M1_LTE_3', 'M1_STE',...
    'M2_LTE_1', 'M2_LTE_2', 'M2_LTE_3', 'M2_STE'};

model_dir = fullfile(pwd,'model');
mkdir(model_dir)

baseName = 'lookup_table_mouse'


D0 = 2.48e-9; % 28 C (https://dtrx.de/od/diff/index.html#arrhenius)
%D0 = 1.5e-9; % just to test
Rs = 1e-6*linspace(1,30,600); %600


switch 1 %1-cylinder, 2-stick, 3-ellipsoid, 4-sphere
    case 1
        auxP.shape = 2; LookupTableName = [baseName '_cylinder'];
    case 2
        auxP.shape = 4; LookupTableName = [baseName '_stick'];
    case 3
        auxP.shape = 3; LookupTableName = [baseName '_ellipsoid'];
    case 4
        auxP.shape = 1; LookupTableName = [baseName '_sphere'];
end


auxP.tortuosity = 0;

% collect relevant waveform info
if save_wfm_common
    
    for n = 1:numel(waveform_names)
        path = fullfile(waveform_dir,[waveform_names{n} '_info.mat']);
        load(path)
        
        wfms(n).trPS_thresh = trPS_thresh;
        
        wfms(n).f0 = wfm.f(max(find(wfm.trPS<trPS_thresh)));
        wfms(n).f_max = max(wfm.f);
        
        ind = find(abs(wfm.f_full)<wfms(n).f0);
        wfms(n).f_full = wfm.f_full(ind);
        PS_full = wfm.PS_full(ind,:,:);
        tmp_trace = PS_full(:,1,1)+PS_full(:,2,2)+PS_full(:,3,3);
        b = sum(tmp_trace);
        wfms(n).trPS_full = tmp_trace/b; % normalize
        wfms(n).PS_full = PS_full/b;% normalize
        
        wfms(n).b_delta = wfm.b_delta;
        wfms(n).TE = wfm.TE;
        wfms(n).name = waveform_names{n};
        
    end
    
    save(fullfile(waveform_dir,'wfms_info.mat'),'wfms')
    
else
    load(fullfile(waveform_dir,'wfms_info.mat'),'wfms')
    
end

R.Rs = Rs;
R.D0 = D0;
R.auxP = auxP;

M = numel(wfms);
N = length(Rs);

if auxP.shape == 3 % ellipsoid
    Parameters = [Rs/5; Rs; D0*ones(1,length(Rs))]';
else
    Parameters = [Rs; D0*ones(1,length(Rs))]';
end

for m = 1:M
    R.b_delta(m) = wfms(m).b_delta;
    R.TE(m) =  wfms(m).TE;
    R.name{m} = wfms(m).name;
    
    for n = 1:N
        %[l1 l2 l3] = fLw([Rs(n),D0],wfms(m),auxP,0);
        [l1 l2 l3] = fLw(Parameters(n,:),wfms(m),auxP,1);
        
        trD = (l1+l2+l3)/3;
        R.MD(m,n) = sum(wfms(m).trPS_full.*trD);
        
        for i = 1:3
            for j = 1:3
                Delta(i,j) =  real(sum(wfms(m).PS_full(:,i,j).*(l3-l1))); % axial symmetry (check!)
            end
        end
        
        R.VA(m,n) = 0;
        for i = 1:3
            for j = 1:3
                R.VA(m,n) =  R.VA(m,n) + 3*Delta(i,j)^2 - Delta(i,i)*Delta(j,j); % axial symmetry and no compartment size/shape dispersion
            end
        end
        
        R.VA(m,n) = 2/45*R.VA(m,n);
        
        
        disp(sprintf('wfm = %d/%d, R = %d/%d',m,M,n,N))
        
    end
    
end

save(fullfile(data_dir,[LookupTableName '.mat']),'R')



