% Calculate and plot signals due to background gradients for cylinder powders as a function of b, Gb and direction.
% Uses attenuation_cylinder.mat file in the *model* folder.

clear all
close all

saveFig = 1;
fig_resolution = '-r300'; % dpi, e.g. '-r72' '-r600'

model_dir = fullfile(pwd,'model');
substrateName = 'attenuation_cylinder';
load(fullfile(model_dir,[substrateName '.mat']),'R')
R.name


tunedLTE_ind = 1+4*[0 1 2]; % tuned
detunedLTE_ind = 3+4*[0 1 2]; % detuned
STE_ind = 4+4*[0 1 2];

col = [1 0 0; ... % tuned
    0 0 1; ... % detuned
    .4 .4 .4]; %STE

P.b = 2e9*linspace(0,1,300);
P.Gb = [30e-3 10e-3 3e-3]; %Gb = 3e-3 in Szczepankiewicz F, Sjölund J: Cross-term-compensated gradient waveform design for tensor-valued diffusion MRI. J Magn Reson 2021; 328:106991.

P.Nb = length(P.b);
P.NGb = length(P.Gb);

P.Nr = length(R.Rs);
P.Nwf = size(R.att_d,1);
P.Ndir = size(R.att_d,3);
P.NGb_dir = size(R.att_db,3);


% signals for different background directions
for nback_dir = 1:P.NGb_dir
    
    att_db = squeeze(R.att_db(:,:,nback_dir,:));
    att_b = squeeze(R.att_b(:,:,nback_dir,:));
    
    for nGb = 1:P.NGb
        Gb = P.Gb(nGb);
        
        for nb = 1:P.Nb
            b = P.b(nb);
            
            E = exp(-b*R.att_d -sqrt(b)*Gb*att_db -Gb^2*att_b);
            
            P.S(:,:,nb,nback_dir,nGb) = mean(E,3);
        end
    end
end


% ---------- show signal vs b ---------------------------------------------

lineStyle = '-';

showLabels = 0;

fig.fs = 16;
fig.lw = 3;

%lws = [8 6 3];
%lws = [4 3 2];
%lws = [4 2 1];
lws = 2*[1 1 1];

GR = 1; %1.62;
fig.H = 400;
fig.W = fig.H*GR;

for M = 0:2
    
    ind = [tunedLTE_ind(1+M); detunedLTE_ind(1+M); STE_ind(1+M)];
    
    switch M %front-back + YLIM
        case 0
            plot_order = [2 3 1];
            YLIM = [.11 1];
        case 1
            plot_order = [1 3 2];
            YLIM = [.08 1];
        case 2
            plot_order = [2 3 1];
            YLIM = [.05 1];
    end
    
    for nGb = 1:length(P.Gb)
        
        fh = figure(1);
        clf
        fh.Color = [1 1 1];
        pause(.1)
        fh.Position = [400 100 fig.W fig.H];
        subFigH = .75;
        subFigW = .21;
        subFigLeft = [.1 .08+[1 2]/3];
        subFigBottom = .15;
        
        
        X = P.b*1e-6;
        
        Nrs = 1:3;
        
        Nr = Nrs(1);
        axes('position',[subFigLeft(1) subFigBottom subFigW subFigH]);
        hold on
        for nwf = plot_order
            Y = squeeze(P.S(ind(nwf),Nr,:,:,nGb));
            plot(X,Y,'LineWidth',lws(nwf),'color',col(nwf,:),'LineStyle',lineStyle)
        end
        set(gca, 'YScale', 'log',...
            'FontSize',fig.fs,'box','off','TickDir','out','LineWidth',fig.lw,'TickLength',[.02 .02])
        
        ylim(YLIM)
        if showLabels
            ylabel('signal')
            xlabel('b [s/mm^2]')
            title(sprintf('R = %.2f um',1e6*R.Rs(Nr)))
        end
        
        Nr = Nrs(2);
        axes('position',[subFigLeft(2) subFigBottom subFigW subFigH]);
        hold on
        for nwf = plot_order
            Y = squeeze(P.S(ind(nwf),Nr,:,:,nGb));
            plot(X,Y,'LineWidth',lws(nwf),'color',col(nwf,:),'LineStyle',lineStyle)
        end
        
        set(gca, 'YScale', 'log',...
            'FontSize',fig.fs,'box','off','TickDir','out','LineWidth',fig.lw,'TickLength',[.02 .02])
        
        ylim(YLIM)
        %     xlim(XLIM)
        
        %     ylabel('signal')
        %     xlabel('b [s/mm^2]')
        if showLabels
            title(sprintf('R = %.2f um',1e6*R.Rs(Nr)))
        end
        
        Nr = Nrs(3);
        axes('position',[subFigLeft(3) subFigBottom subFigW subFigH]);
        hold on
        for nwf = plot_order
            Y = squeeze(P.S(ind(nwf),Nr,:,:,nGb));
            plot(X,Y,'LineWidth',lws(nwf),'color',col(nwf,:),'LineStyle',lineStyle)
        end
        set(gca, 'YScale', 'log',...
            'FontSize',fig.fs,'box','off','TickDir','out','LineWidth',fig.lw,'TickLength',[.02 .02])
        %     ylabel('signal')
        %     xlabel('b [s/mm^2]')
        ylim(YLIM)
        %     xlim(XLIM)
        if showLabels
            title(sprintf('R = %.2f um',1e6*R.Rs(Nr)))
        end
        %lh = legend();
        % lh.Box = 'off';
        % lh.String = {'', '', '',''};
        
        
        if saveFig
            out_folder = 'add_test';
            mkdir(out_folder)
            fig_path = fullfile(out_folder,sprintf('test_background_Svsb_R%d_%d_%d_M%d_Gb_%d_raw.png', ...
                round(1e7*R.Rs(Nrs(1))), round(1e7*R.Rs(Nrs(2))), round(1e7*R.Rs(Nrs(3))), M, round(P.Gb(nGb)*1e3) ));
            
            print(fh,fig_path,'-dpng',fig_resolution);
        end
        
        %return
    end
    
end


