% Calculate normalized encoding spectra by inluding the effects of background gradients.
% Saves results in wfms_background_info.mat in the *waveform* folder.

clear all
close all

% -------------------------- SELECT WAVEFORMS -------------------------------------------------------------

waveform_dir = 'wfm_preclinical';
waveform_names = {'M0_LTE_1','M0_LTE_2', 'M0_LTE_3', 'M0_STE', ...
    'M1_LTE_1','M1_LTE_2','M1_LTE_3','M1_STE',...
    'M2_LTE_1','M2_LTE_2','M2_LTE_3','M2_STE',...
    'PGSE_LTE'};

% waveform_names = {'M2_LTE_1','M2_LTE_2','M2_LTE_3','M2_STE'};

dt = 20e-6;

addpath(genpath(fullfile(pwd,'add')));

gmr = 26.75e7;

encoding_time = 0;

% FT info - for analysis
N0 = 1e7; %1e7; % 0-padding


Gb = 1; %Gb = 3e-3 in Szczepankiewicz F, Sjölund J: Cross-term-compensated gradient waveform design for tensor-valued diffusion MRI. J Magn Reson 2021; 328:106991.

% lnE(b) = - b*sd:D - sqrt(b)*sdb:D - sb:D
% g is scaled to yield b = 1, so sqrt(b) is the gradient scaling and Gb is background gradient scaling
% sum(sd) = 1 is ensured by rescaling spectra
% the same rescaling is used for sdb and sb
% lnE(b) = - b*sd:D - sqrt(b)*Gb*sdb:D - Gb^2*sb:D

% gb_dir = [1 0 0; ...
%           0 1 0; ...
%           0 0 1; ...
%           1 1 0; ...
%           1 0 1; ...
%           0 1 1];
% gb_dir = [gb_dir; -gb_dir];
%
% for n = 1:length(gb_dir)
%     gb_dir(n,:) = gb_dir(n,:)/norm(gb_dir(n,:));
% end

load UDSRTriN50
gb_dir = [UDSR.x, UDSR.y, UDSR.z];


Nb_dir = length(gb_dir);

power_thresh = 1-1e-4; % limit frequency window


for count = 1:numel(waveform_names)
    
    waveform_name = waveform_names{count};
    
    load(fullfile(waveform_dir,[waveform_name '.mat']))
    
    Nwave = size(g,1);
    
    if (encoding_time == 0) & (dt > 0)
        TE = Nwave*dt; %duration of the waveform without 0-padding
        t = linspace(0,TE,Nwave)';
    elseif (encoding_time > 0) & (dt == 0)
        dt = encoding_time/Nwave;
        TE = encoding_time; %duration of the waveform without 0-padding
        t = linspace(0,TE,Nwave)';
    else % do interpolation
        Nwave_new = round(encoding_time/dt);
        TE = Nwave_new*dt; %duration of the waveform without 0-padding
        
        %interpolate
        tmp = linspace(0,TE,Nwave)';
        t = linspace(0,TE,Nwave_new)';
        g = interp1(tmp,g,t);
        Nwave = Nwave_new;
    end
    
    q = gmr*cumsum(g)*dt;
    B = q'*q*dt;
    [V, L] = eig(B);
    b = trace(B);
    
    % normalize waveforms for b = 1
    norm_const = sqrt(1/b);
    g = g*norm_const;
    q = q*norm_const;
    
    
    [f_full, PS_full] = fCrossPower(q,dt,N0);
    
    
    % limit frequency window
    
    s = cumsum(PS_full(:,1,1) + PS_full(:,2,2) +PS_full(:,3,3));
    s = s/s(end);
    thresh_ind = find(s > 1-power_thresh & s < power_thresh);
    
    PS_full = PS_full(thresh_ind,:,:);
    f_full = f_full(thresh_ind);
    
    %     df = diff(f_full(1:2));
    %     bt = squeeze(sum(PS_full)*df);
    %     real(bt), imag(bt)
    
    % scale (b = 1)
    bt = squeeze(sum(PS_full));
    scale_PS = trace(bt);
    PS_full = PS_full/scale_PS;
    
    wfms(count).f_full = f_full;
    wfms(count).PS_full = PS_full;
    wfms(count).name = waveform_name;
    
    %     k = 1; l = 1; ps = squeeze(PS_full(:,k,l));
    %     figure(1),clf, plot(f_full,real(ps),'b-',f_full,imag(ps),'r-')
    
    % background gradient
    h180 = -sign(t-TE/2); % dephasing direction (flips after 180)
    
    
    for nb = 1:Nb_dir
        display([count/numel(waveform_names) nb/Nb_dir])
        qb = gmr*cumsum(h180*gb_dir(nb,:))*dt; % Gb = 1 used here (needs scaling)
        
        %         figure(1),clf, hold on
        %         plot(qb)
        
        % mixed terms
        PSdb = fCrossPowerMixed(q,qb,N0);
        % limit frequency window
        PSdb = PSdb(thresh_ind,:,:);
        
        % scale
        PSdb = PSdb/scale_PS;
        
        %      q'*qb*dt
        %      tmp = squeeze(sum(PSdb));
        %      real(tmp)
        %      imag(tmp)
        %      trace(tmp)
        
        % background only
        [~, PSb] = fCrossPower(qb,dt,N0);
        
        % limit frequency window
        PSb = PSb(thresh_ind,:,:);
        
        % scale
        PSb = PSb/scale_PS;
        
        
        % traces
        bdb = trace(squeeze(sum(PSdb)));
        bb = trace(squeeze(sum(PSb)));
        
        
        wfms(count).PSdb(nb,:,:,:) = PSdb;
        wfms(count).PSb(nb,:,:,:) = PSb;
        wfms(count).traces(nb,:) = [1, bdb, bb];
        
    end
    
end

save(fullfile(waveform_dir,'wfms_background_info.mat'),'wfms','-v7.3')


%--------------------------------- FUNCTIONS ------------------------------

function [f, PS] = fCrossPower(q,dt,N_zeropad)
%  cross-power spectra: Fi*conj(Fj)

% zero padding
q = [zeros(N_zeropad,3); q; zeros(N_zeropad,3)];

N = size(q,2);
NT = size(q,1);

f = 1/dt*(0:(NT/2))/NT;
f = [-fliplr(f(2:end)) f(1:end)];
PS = zeros(NT+1,N,N);

FT = fft(q);
conjFT = conj(FT);

% F = fftshift(FT(:,2));
% figure(1),clf, hold on
% plot(real(F)), plot(imag(F))

for m = 1:N
    for n = 1:N
        ps = fftshift(FT(:,m).*conjFT(:,n));
        PS(:,m,n) = [ps; ps(1)];
    end
end

end


function PS = fCrossPowerMixed(q1,q2,N_zeropad)
%  cross-power spectra: F1i*conj(F2j) + F2i*conj(F1j)

% zero padding
q1 = [zeros(N_zeropad,3); q1; zeros(N_zeropad,3)];
q2 = [zeros(N_zeropad,3); q2; zeros(N_zeropad,3)];

FT1 = fft(q1);
conjFT1 = conj(FT1);
FT2 = fft(q2);
conjFT2 = conj(FT2);

N = size(q1,2);
for m = 1:N
    for n = 1:N
        ps = fftshift(FT1(:,m).*conjFT2(:,n) + FT2(:,m).*conjFT1(:,n));
        PS(:,m,n) = [ps; ps(1)];
    end
end

end


